<?php
/**
 * @package    JGive
 *
 * @author     Techjoomla <extensions@techjoomla.com>
 * @copyright  Copyright (C) 2009 - 2018 Techjoomla. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

// No direct access
defined('_JEXEC') or die('Restricted access');

// Component Helper
jimport('joomla.application.component.helper');
jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');

/**
 * Social integration lib
 *
 * @package     JGive
 * @subpackage  com_jgive
 * @since       1.0
 */
class Activitysocialintegrationprofiledata
{
	/**
	 * Function to get user profile URL
	 *
	 * @param   STRING  $integration_option  option
	 * @param   INT     $userid              user id
	 *
	 * @return URL
	 */
	public function getUserProfileUrl($integration_option, $userid)
	{
		$link = '';

		if ($integration_option == 'Joomla')
		{
			$link = '';
		}
		elseif ($integration_option == 'Community Builder')
		{
			$cbpath = JPATH_ROOT . '/' . 'components' . '/' . 'com_community';

			if ($cbpath)
			{
				$baseUrl = 'index.php?option=com_comprofiler&task=userprofile&user=';
				$link = JUri::root() . substr(JRoute::_($baseUrl . $userid . '&Itemid=' . $itemid), strlen(JUri::base(true)) + 1);
			}
		}
		elseif ($integration_option == 'JomSocial')
		{
			$jspath = JPATH_ROOT . '/' . 'components' . '/' . 'com_community';

			if ($jspath)
			{
				$link = '';

				if (file_exists($jspath))
				{
					include_once $jspath . '/' . 'libraries' . '/' . 'core.php';

					$link = JUri::root() . substr(CRoute::_('index.php?option=com_community&view=profile&userid=' . $userid), strlen(JUri::base(true)) + 1);
				}
			}
		}
		elseif ($integration_option == 'Jomwall')
		{
			$awdpath = JPATH_ROOT . '/' . 'components' . '/' . 'com_awdwall';

			if ($awdpath)
			{
				if (!class_exists('AwdwallHelperUser'))
				{
					require_once JPATH_SITE . '/' . 'components' . '/' . 'com_awdwall' . '/' . 'helpers' . '/' . 'user.php';
				}

				$awduser = new AwdwallHelperUser;
				$Itemid  = $awduser->getComItemId();
				$link    = JRoute::_('index.php?option=com_awdwall&view=awdwall&layout=mywall&wuid=' . $userid . '&Itemid=' . $Itemid);
			}
		}
		elseif ($integration_option == 'EasySocial')
		{
			$espath = JPATH_ROOT . '/' . 'components' . '/' . 'com_easysocial';

			if ($espath)
			{
				$link = '';

				if (file_exists($espath))
				{
					require_once JPATH_ADMINISTRATOR . '/components/com_easysocial/includes/foundry.php';
					$user = Foundry::user($userid);
					$link = JRoute::_($user->getPermalink());
				}
			}
		}

		return $link;
	}

	/**
	 * Function to get user avatar
	 *
	 * @param   STRING  $integration_option  option
	 * @param   OBJECT  $user                user object
	 *
	 * @return URL
	 */
	public function getUserAvatar($integration_option, $user)
	{
		$activitysocialintegrationprofiledata = new activitysocialintegrationprofiledata;
		$useremail                            = $user->email;
		$userid                               = $user->id;
		$uimage                               = '';

		if ($integration_option == "Joomla")
		{
			$uimage = $activitysocialintegrationprofiledata->get_gravatar($useremail, '40', 'mm', 'g');
		}
		elseif ($integration_option == "Community Builder")
		{
			$installed = $activitysocialintegrationprofiledata->Checkifinstalled('com_comprofiler');

			if ($installed)
			{
				$uimage = $activitysocialintegrationprofiledata->getCBUserAvatar($userid);
			}
		}
		elseif ($integration_option == "JomSocial")
		{
			$installed = $activitysocialintegrationprofiledata->Checkifinstalled('com_community');

			if ($installed)
			{
				$uimage = $activitysocialintegrationprofiledata->getJomsocialUserAvatar($userid);
			}
		}
		elseif ($integration_option == "Jomwall")
		{
			$installed = $activitysocialintegrationprofiledata->Checkifinstalled('com_awdwall');

			if ($installed)
			{
				$uimage = $activitysocialintegrationprofiledata->getJomwallUserAvatar($userid);
			}
		}
		elseif ($integration_option == "EasySocial")
		{
			$installed = $activitysocialintegrationprofiledata->Checkifinstalled('com_easysocial');

			if ($installed)
			{
				$uimage = $activitysocialintegrationprofiledata->getEasySocialUserAvatar($userid);
			}
		}

		return $uimage;
	}

	/**
	 * Function to check if component is installed
	 *
	 * @param   STRING  $folder  folder name
	 *
	 * @return URL
	 */
	public function Checkifinstalled($folder)
	{
		$path = JPATH_SITE . '/' . 'components' . '/' . $folder;

		if (JFolder::exists($path))
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Get ES user avatar
	 *
	 * @param   INT  $userid  user id
	 *
	 * @return URL
	 */
	public function getEasySocialUserAvatar($userid)
	{
		require_once JPATH_ADMINISTRATOR . '/components/com_easysocial/includes/foundry.php';
		$user   = Foundry::user($userid);
		$uimage = $user->getAvatar();

		return $uimage;
	}

	/**
	 * Get CB user avatar
	 *
	 * @param   INT  $userid  user id
	 *
	 * @return URL
	 */
	public function getCBUserAvatar($userid)
	{
		$db = JFactory::getDbo();
		$q  = "SELECT a.id,a.username,a.name, b.avatar, b.avatarapproved
			FROM #__users a, #__comprofiler b
			WHERE a.id=b.user_id AND a.id=" . $userid;
		$db->setQuery($q);
		$user     = $db->loadObject();
		$img_path = JUri::root() . "images/comprofiler";

		if (isset($user->avatar) && isset($user->avatarapproved))
		{
			if (substr_count($user->avatar, "/") == 0)
			{
				$uimage = $img_path . '/tn' . $user->avatar;
			}
			else
			{
				$uimage = $img_path . '/' . $user->avatar;
			}
		}
		elseif (isset($user->avatar))
		{
			$uimage = JUri::root() . "/components/com_comprofiler/plugin/templates/default/images/avatar/nophoto_n.png";
		}
		else
		{
			$uimage = JUri::root() . "/components/com_comprofiler/plugin/templates/default/images/avatar/nophoto_n.png";
		}

		return $uimage;
	}

	/**
	 * Get jomsocial user avatar
	 *
	 * @param   INT  $userid  user id
	 *
	 * @return URL
	 */
	public function getJomsocialUserAvatar($userid)
	{
		$mainframe = JFactory::getApplication();
		/*included to get jomsocial avatar*/
		$uimage    = '';
		$jspath    = JPATH_ROOT . '/' . 'components' . '/' . 'com_community';

		if (file_exists($jspath))
		{
			include_once $jspath . '/' . 'libraries' . '/' . 'core.php';

			$user   = CFactory::getUser($userid);
			$uimage = $user->getThumbAvatar();

			if (!$mainframe->isSite())
			{
				$uimage = str_replace('administrator/', '', $uimage);
			}
		}

		return $uimage;
	}

	/**
	 * Get jomwall user avatar
	 *
	 * @param   INT  $userid  user id
	 *
	 * @return URL
	 */
	public function getJomwallUserAvatar($userid)
	{
		if (!class_exists('AwdwallHelperUser'))
		{
			require_once JPATH_SITE . '/' . 'components' . '/' . 'com_awdwall' . '/' . 'helpers' . '/' . 'user.php';
		}

		$awduser = new AwdwallHelperUser;
		$uimage  = $awduser->getAvatar($userid);

		return $uimage;
	}

	/**
	 * Function to get gravatar
	 *
	 * @param   STRING  $email  email id
	 * @param   INT     $s      size
	 * @param   STRING  $d      d
	 * @param   STRING  $r      r
	 *
	 * @return URL
	 */
	public function get_gravatar($email, $s = 80, $d = 'mm', $r = 'g')
	{
		$url = 'https://www.gravatar.com/avatar/';
		$url .= md5(strtolower(trim($email)));
		$url .= "?s=$s&d=$d&r=$r";

		return $url;
	}
}
