<?php
/**
 * @package    JGive_Campaigns
 * @author     Techjoomla <extensions@techjoomla.com>
 * @copyright  Copyright (C) 2009-2017 Techjoomla. All rights reserved.
 * @license    GNU/GPL v2 http://www.gnu.org/licenses/gpl-2.0.html
 *
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

/* Load language file*/
$lang = JFactory::getLanguage();
$lang->load('plg_jgivecampaigns', JPATH_ADMINISTRATOR, $lang->getTag(), true);
$lang->load('com_jgive', JPATH_ADMINISTRATOR, $lang->getTag(), true);

// Include TJReport Model
JLoader::import('com_tjreports.models.reports', JPATH_SITE . '/components');

/**
 * Campaigns report plugin of TJReport
 *
 * @since  1.0.0
 */
class TjreportsModelJgivecampaigns extends TjreportsModelReports
{
	protected $default_order = 'start_date';

	protected $default_order_dir = 'DESC';

	/**
	 * Constructor.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @see     JModelLegacy
	 * @since   1.6
	 */
	public function __construct($config = array())
	{
		$this->columns = array(
			'title' => array('table_column' => 'c.title', 'title' => 'PLG_TJREPORTS_JGIVE_CAMPAIGNS_CAMPAIGN_TITLE'),
			'promotername' => array('table_column' => 'u.name', 'title' => 'PLG_TJREPORTS_JGIVE_CAMPAIGNS_PROMOTER_NAME'),
			'published' => array('table_column' => 'c.published', 'title' => 'PLG_TJREPORTS_JGIVE_CAMPAIGNS_PUBLISHED_STATUS'),
			'start_date' => array('table_column' => 'c.start_date', 'title' => 'PLG_TJREPORTS_JGIVE_CAMPAIGNS_START_DATE'),
			'end_date' => array('table_column' => 'c.end_date', 'title' => 'PLG_TJREPORTS_JGIVE_CAMPAIGNS_END_DATE'),
			'goal_amount' => array('table_column' => 'c.goal_amount', 'title' => 'PLG_TJREPORTS_JGIVE_CAMPAIGNS_GOAL_AMOUNT'),
			'numberOfDonors' => array('title' => 'PLG_TJREPORTS_JGIVE_CAMPAIGNS_NUMBER_OF_DONORS', 'disable_sorting' => true),
			'amountReceived' => array('title' => 'PLG_TJREPORTS_JGIVE_CAMPAIGNS_AMOUNT_RECIEVED', 'disable_sorting' => true)
		);

		parent::__construct($config);
	}

	/**
	 * Get client of this plugin
	 *
	 * @return array<string,mixed|string> Plugin Details
	 *
	 * @since   2.0
	 * */
	public function getPluginDetail()
	{
		return $detail = array('client' => 'com_jgive', 'title' => JText::_('PLG_TJREPORTS_JGIVECAMPAIGNS'));
	}

	/**
	 * Create an array of filters
	 *
	 * @return    ARRAY Filters used in reports
	 *
	 * @since    1.0
	 */
	public function displayFilters()
	{
		JLoader::import('components.com_jgive.helpers.tjreports', JPATH_SITE);
		$jGiveTjreportsHelper = new JGiveTjreportsHelper;
		$campaignsListArray = $jGiveTjreportsHelper->getAllCampaigns();
		$allCampaignsList = $jGiveTjreportsHelper->getFilterOptions($campaignsListArray);

		$promoterListArray = $jGiveTjreportsHelper->getCampaignsPromoter();
		$promotersList = $jGiveTjreportsHelper->getFilterOptions($promoterListArray);

		$statusList = array();
		$statusList[] = JHTML::_('select.option', '', JText::_('PLG_TJREPORTS_JGIVE_FILTER_SELECT_OPTION'));
		$statusList[] = JHTML::_('select.option', 1, JText::_('YES'));
		$statusList[] = JHTML::_('select.option', 0, JText::_('NO'));

		$dispFilters = array(
			array(
				'title' => array(
					'search_type' => 'select', 'select_options' => $allCampaignsList,'type' => 'equal', 'searchin' => 'c.id'
					),
				'promotername' => array(
					'search_type' => 'select', 'select_options' => $promotersList, 'searchin' => 'u.id'
					),
				'published' => array(
					'search_type' => 'select', 'select_options' => $statusList, 'type' => 'equal', 'searchin' => 'c.published'
					),
				'start_date' => array(
					'search_type' => 'calendar',
					'searchin' => 'c.start_date > %s',
					'type' => 'custom',
					'start_date' => array('attrib' => array('onChange' => 'tjrContentUI.report.attachCalSubmit(this);', 'placeholder' => 'FROM (YYYY-MM-DD)')),
					),
				'end_date' => array(
					'search_type' => 'calendar',
					'searchin' => 'c.end_date < %s',
					'type' => 'custom',
					'end_date' => array('attrib' => array('placeholder' => 'FROM (YYYY-MM-DD)','onChange' => 'tjrContentUI.report.attachCalSubmit(this);')),
					)
			)
		);

		return $dispFilters;
	}

	/**
	 * Method to get a JDatabaseQuery object for retrieving the data set from a database.
	 *
	 * @return  JDatabaseQuery  A JDatabaseQuery object to retrieve the data set.
	 *
	 * @since   1.6
	 */
	protected function getListQuery()
	{
		$db        = $this->_db;
		$query     = parent::getListQuery();
		$query->select('SUM(o.amount) AS amountReceived');
		$query->select('COUNT(o.campaign_id) AS numberOfDonors');
		$query->from($db->quoteName('#__jg_campaigns', 'c'));
		$query->join('LEFT', $db->quoteName('#__users', 'u') . ' ON (' . $db->quoteName('c.creator_id') . ' = ' . $db->quoteName('u.id') . ')');
		$query->join(
		'LEFT', $db->quoteName('#__jg_orders', 'o') . ' ON (' . $db->quoteName('o.campaign_id') . ' = ' . $db->quoteName('c.id') .
		' AND ' . $db->quoteName('o.status') . ' = ' . $db->quote('C') . ')'
		);
		$query->join('LEFT', $db->quoteName('#__jg_donors', 'd') . ' ON (' . $db->quoteName('d.id') . ' = ' . $db->quoteName('o.donor_id') . ')');
		$query->where($db->quoteName('c.creator_id') . ' = ' . $db->quoteName('u.id'));
		$query->group($db->quoteName('c.id'));

		return $query;
	}

	/**
	 * Method to get an array of data items.
	 *
	 * @return  mixed  An array of data items on success, false on failure.
	 *
	 * @since   1.6
	 */
	public function getItems()
	{
		$newItems = array();
		$items = parent::getItems();

		JLoader::import('components.com_jgive.helper', JPATH_SITE);
		$jGiveFrontendHelper = new JgiveFrontendHelper;

		$params = JComponentHelper::getParams('com_jgive');

		foreach ($items as $item)
		{
			if (isset($item['title']))
			{
				$item['title'] = htmlspecialchars($item['title'], ENT_COMPAT, 'UTF-8');
			}

			if (isset($item['goal_amount']))
			{
				$item['goal_amount'] = $jGiveFrontendHelper->getFormattedPrice($item['goal_amount']);
			}

			$item['amountReceived'] = $item['amountReceived'] ? $jGiveFrontendHelper->getFormattedPrice($item['amountReceived']) : "-";

			if (isset($item['start_date']))
			{
				$item['start_date'] = JHtml::_('date', $item['start_date'], $params->get('date_format', 'j  M  Y'));
			}

			if (isset($item['end_date']))
			{
				$item['end_date'] = JHtml::_('date', $item['end_date'], $params->get('date_format', 'j  M  Y'));
			}

			if (isset($item['published']))
			{
				$item['published'] = JText::_("PLG_TJREPORTS_JGIVE_CAMPAIGNS_PUBLISHED_STATUS_YES");
			}
			else
			{
				$item['published'] = JText::_("PLG_TJREPORTS_JGIVE_CAMPAIGNS_PUBLISHED_STATUS_NO");
			}

			$newItems[] = $item;
		}

		return $newItems;
	}
}
