<?php
/**
 * @package    JGive_CampaignsPromoter
 * @author     Techjoomla <extensions@techjoomla.com>
 * @copyright  Copyright (C) 2009-2017 Techjoomla. All rights reserved.
 * @license    GNU/GPL v2 http://www.gnu.org/licenses/gpl-2.0.html
 *
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

/* Load language file*/
$lang = JFactory::getLanguage();
JFactory::getLanguage()->load('plg_jgivecampaignspromoter', JPATH_ADMINISTRATOR, $lang->getTag(), true);

// Include TJReport Model
JLoader::import('com_tjreports.models.reports', JPATH_SITE . '/components');

/**
 * Campaigns report plugin of TJReport
 *
 * @since  1.0.0
 */
class TjreportsModelJgivecampaignspromoter extends TjreportsModelReports
{
	protected $default_order = 'u.name';

	protected $default_order_dir = 'ASC';

	/**
	 * Constructor.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @see     JModelLegacy
	 * @since   1.6
	 */
	public function __construct($config = array())
	{
		$this->columns = array(
			'promotername' => array('table_column' => 'u.name', 'title' => 'PLG_TJREPORTS_JGIVE_CAMPAIGNS_PROMOTER_PROMOTER_NAME'),
			'promoteremail' => array('table_column' => 'u.email', 'title' => 'PLG_TJREPORTS_JGIVE_CAMPAIGNS_PROMOTER_PROMOTER_EMAIL'),
			'promoterphone' => array('table_column' => 'c.phone', 'title' => 'PLG_TJREPORTS_JGIVE_CAMPAIGNS_PROMOTER_PHONE_NUMBER'),
			'no_of_campaigns' => array('table_column' => 'c.id', 'title' => 'PLG_TJREPORTS_JGIVE_CAMPAIGNS_PROMOTER_NUMBER_OF_CAMPAIGNS'),
			'total_amount_raised' => array('table_column' => 'c.goal_amount', 'title' => 'PLG_TJREPORTS_JGIVE_CAMPAIGNS_PROMOTER_TOTAL_AMOUNT_RAISED')
		);

		parent::__construct($config);
	}

	/**
	 * Get client of this plugin
	 *
	 * @return array<string,mixed|string> Plugin Details
	 *
	 * @since   2.0
	 * */
	public function getPluginDetail()
	{
		return $detail = array('client' => 'com_jgive', 'title' => JText::_('PLG_TJREPORTS_JGIVECAMPAIGNSPROMOTER'));
	}

	/**
	 * Create an array of filters
	 *
	 * @return    ARRAY Filters used in reports
	 *
	 * @since    1.0
	 */
	public function displayFilters()
	{
		$dispFilters = array(
			array(
				'promotername' => array(
					'search_type' => 'text', 'searchin' => 'u.name'),
				'promoteremail' => array(
					'search_type' => 'text', 'searchin' => 'u.email'),
			)
		);

		return $dispFilters;
	}

	/**
	 * Method to get a JDatabaseQuery object for retrieving the data set from a database.
	 *
	 * @return  JDatabaseQuery  A JDatabaseQuery object to retrieve the data set.
	 *
	 * @since   1.6
	 */
	protected function getListQuery()
	{
		$db        = $this->_db;
		$query     = parent::getListQuery();
		$colToshow = (array) $this->getState('colToshow');
		$filters = $this->getState('filters');

		$query->select('COUNT(c.id) as no_of_campaigns');
		$query->select('SUM(c.goal_amount) as total_amount_raised');

		$query->from($db->quoteName('#__jg_campaigns', 'c'));
		$query->join('LEFT', $db->quoteName('#__users', 'u') . ' ON (' . $db->quoteName('c.creator_id') . ' = ' . $db->quoteName('u.id') . ')');
		$query->group($db->quoteName('c.creator_id'));

		return $query;
	}

	/**
	 * Method to get an array of data items.
	 *
	 * @return  mixed  An array of data items on success, false on failure.
	 *
	 * @since   1.6
	 */
	public function getItems()
	{
		$newItems = array();
		$items = parent::getItems();

		JLoader::import('components.com_jgive.helper', JPATH_SITE);
		$jGiveFrontendHelper = new JgiveFrontendHelper;

		foreach ($items as $item)
		{
			$item['total_amount_raised'] = $jGiveFrontendHelper->getFormattedPrice($item['total_amount_raised']);
			$item['promoterphone'] = !empty($item['promoterphone'])	 ? $item['promoterphone'] : "-";
			$newItems[] = $item;
		}

		return $newItems;
	}
}
