<?php
/**
 * @package    JGive_DonorCampaigns
 * @author     Techjoomla <extensions@techjoomla.com>
 * @copyright  Copyright (C) 2009-2017 Techjoomla. All rights reserved.
 * @license    GNU/GPL v2 http://www.gnu.org/licenses/gpl-2.0.html
 *
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

/* Load language file*/
$lang = JFactory::getLanguage();
$lang->load('com_jgive', JPATH_ADMINISTRATOR, $lang->getTag(), true);
$lang->load('plg_jgivedonorcampaign', JPATH_ADMINISTRATOR, $lang->getTag(), true);

// Include TJReport Model
JLoader::import('com_tjreports.models.reports', JPATH_SITE . '/components');

/**
 * Campaigns report plugin of TJReport
 *
 * @since  1.0.0
 */
class TjreportsModelJgivedonorcampaign extends TjreportsModelReports
{
	protected $default_order     = 'name';

	protected $default_order_dir = 'ASC';

	public  $columns             = array();

	/**
	 * Constructor.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @see     JModelLegacy
	 * @since   1.6
	 */
	public function __construct($config = array())
	{
		$this->columns = array(
			'donorname' => array('table_column' => 'd.first_name', 'title' => 'PLG_TJREPORTS_JGIVE_DONOR_CAMPAIGN_DONOR_FIRSTNAME', 'disable_sorting' => false),
			'lastname' => array('table_column' => 'd.last_name', 'title' => 'PLG_TJREPORTS_JGIVE_DONOR_CAMPAIGN_DONOR_LASTNAME'),
			'donoremail' => array('table_column' => 'd.email', 'title' => 'PLG_TJREPORTS_JGIVE_DONOR_CAMPAIGN_DONOR_EMAIL'),
			'campaignname' => array('table_column' => 'c.title', 'title' => 'COM_JGIVE_CAMPAIGN_NAME'),
			'address' => array('table_column' => 'd.address', 'title' => 'PLG_TJREPORTS_JGIVE_DONOR_CAMPAIGN_DONOR_ADDRESS'),
			'address2' => array('table_column' => 'd.address2', 'title' => 'PLG_TJREPORTS_JGIVE_DONOR_CAMPAIGN_DONOR_ADDRESS2'),
			'donorphone' => array('table_column' => 'd.phone', 'title' => 'PLG_TJREPORTS_JGIVE_DONOR_CAMPAIGN_PHONE_NUMBER'),
			'isAnonymous' => array('table_column' => 'dt.annonymous_donation', 'title' => 'PLG_TJREPORTS_JGIVE_DONOR_CAMPAIGN_DONOR_IS_ANONYMOUS'),
			'amountdonated' => array('table_column' => 'o.amount', 'title' => 'PLG_TJREPORTS_JGIVE_DONOR_CAMPAIGN_DONOR_INVESTOR_PAID_AMOUNT'),
			'createdon' => array('table_column' => 'o.cdate', 'title' => 'PLG_TJREPORTS_JGIVE_DONOR_CAMPAIGN_DONATION_CREATED_DATE'),
			'receiveddonation' => array('table_column' => 'o.payment_received_date', 'title' => 'PLG_TJREPORTS_JGIVE_PAYMENT_RECEIVED_DATE'),
			'givebackitem' => array('table_column' => 'g.description', 'title' => 'PLG_TJREPORTS_JGIVE_DONOR_CAMPAIGN_GIVEBACK_ITEM_NAME'),
			'donor_type' => array('table_column' => 'd.donor_type', 'title' => 'PLG_TJREPORTS_JGIVE_DONORSREPORT_DONOR_TYPE')
		);

		parent::__construct($config);
	}

	/**
	 * Get client of this plugin
	 *
	 * @return array<string,mixed|string> Plugin Details
	 *
	 * @since   2.0
	 * */
	public function getPluginDetail()
	{
		return $detail = array('client' => 'com_jgive', 'title' => JText::_('PLG_TJREPORTS_JGIVEDONORCAMPAIGN'));
	}

	/**
	 * Create an array of filters
	 *
	 * @return    ARRAY Filters used in reports
	 *
	 * @since    1.0
	 */
	public function displayFilters()
	{
		JLoader::import('components.com_jgive.helpers.tjreports', JPATH_SITE);
		$jGiveTjreportsHelper = new JGiveTjreportsHelper;
		$campaignsListArray = $jGiveTjreportsHelper->getAllCampaigns();
		$allCampaignsList = $jGiveTjreportsHelper->getFilterOptions($campaignsListArray);

		$donorListArray = $jGiveTjreportsHelper->getDonors();
		$donorOptions = $jGiveTjreportsHelper->getFilterOptions($donorListArray);

		$dispFilters = array(
			array(
				'donorname' => array(
					'search_type' => 'select', 'select_options' => $donorOptions, 'searchin' => 'd.first_name'
					),
				'campaignname' => array(
					'search_type' => 'select', 'select_options' => $allCampaignsList, 'type' => 'equal', 'searchin' => 'c.id'
					),
				'donoremail' => array(
					'search_type' => 'text', 'searchin' => 'd.email')
			),
			array(
				'o.cdate' => array(
					'search_type' => 'date.range',
					'searchin' => 'o.cdate',
					'o.cdate_from' => array('attrib' => array('placeholder' => 'FROM (YYYY-MM-DD)')),
					'o.cdate_to' => array('attrib' => array('placeholder' => 'TO (YYYY-MM-DD)')),
				)
			)
		);

		return $dispFilters;
	}

	/**
	 * Method to get a JDatabaseQuery object for retrieving the data set from a database.
	 *
	 * @return  JDatabaseQuery  A JDatabaseQuery object to retrieve the data set.
	 *
	 * @since   1.6
	 */
	protected function getListQuery()
	{
		$db        = $this->_db;
		$query     = parent::getListQuery();

		$query->from($db->quoteName('#__jg_donors', 'd'));
		$query->join('INNER', $db->quoteName('#__jg_campaigns', 'c') . ' ON (' . $db->quoteName('c.id') . ' = ' . $db->quoteName('d.campaign_id') . ')');
		$query->join('LEFT', $db->quoteName('#__jg_orders', 'o') . ' ON (' . $db->quoteName('d.id') . ' = ' . $db->quoteName('o.donor_id') . ')');
		$query->join('LEFT', $db->quoteName('#__jg_donations', 'dt') . ' ON (' . $db->quoteName('dt.id') . ' = ' . $db->quoteName('d.id') . ')');
		$query->join(
		'LEFT', $db->quoteName('#__jg_campaigns_givebacks', 'g') . ' ON (' . $db->quoteName('g.id') . ' = ' . $db->quoteName('dt.giveback_id') . ')'
		);
		$query->where($db->quoteName('o.status') . '=' . $db->quote('C'));

		return $query;
	}

	/**
	 * Method to get an array of data items.
	 *
	 * @return  mixed  An array of data items on success, false on failure.
	 *
	 * @since   1.6
	 */
	public function getItems()
	{
		$newItems = array();
		$items = parent::getItems();

		jimport('techjoomla.common');
		$tjCommon = new TechjoomlaCommon;

		JLoader::import('components.com_jgive.helper', JPATH_SITE);
		$jGiveFrontendHelper = new JgiveFrontendHelper;

		$params = JComponentHelper::getParams('com_jgive');

		if (is_array($items))
		{
			foreach ($items as $item)
			{
				if (isset($item['campaignname']))
				{
					$item['campaignname'] = htmlspecialchars($item['campaignname'], ENT_COMPAT, 'UTF-8');
				}

				$item['lastname']     = !empty($item['lastname']) ? $item['lastname'] : "-";
				$item['address']      = !empty($item['address']) ? $item['address'] : "-";
				$item['address2']     = !empty($item['address2']) ? $item['address2'] : "-";
				$item['donorphone']   = !empty($item['donorphone']) ? $item['donorphone'] : "-";
				$item['givebackitem'] = !empty($item['givebackitem']) ? $item['givebackitem'] : "-";

				if (empty($item['createdon']) || $item['createdon'] == '0000-00-00 00:00:00')
				{
					$item['createdon'] = ' - ';
				}
				else
				{
					$item['createdon'] = $tjCommon->getDateInLocal($item['createdon'], 0, $params->get('date_format', 'j  M  Y'));
				}

				$item['isAnonymous'] = isset($items['isAnonymous'])? JText::_(
				'PLG_TJREPORTS_JGIVE_DONOR_CAMPAIGN_DONOR_ISANONYMOUS_YES'): JText::_('PLG_TJREPORTS_JGIVE_DONOR_CAMPAIGN_DONOR_ISANONYMOUS_NO');

				if (isset($item['amountdonated']))
				{
					$item['amountdonated'] = $jGiveFrontendHelper->getFormattedPrice($item['amountdonated']);
				}

				if (isset($item['donor_type']) && $item['donor_type'] == "org")
				{
					$item['donor_type'] = JText::_('PLG_TJREPORTS_JGIVE_ORGANIZATION');
				}
				else
				{
					$item['donor_type'] = JText::_('PLG_TJREPORTS_JGIVE_INDIVIDUAL');
				}

				if (empty($item['receiveddonation']) || $item['receiveddonation'] == '0000-00-00 00:00:00')
				{
					$item['receiveddonation'] = ' - ';
				}
				else
				{
					$item['receiveddonation'] = $tjCommon->getDateInLocal($item['receiveddonation'], 0, $params->get('date_format', 'j  M  Y'));
				}

				$newItems[] = $item;
			}
		}

		return $newItems;
	}
}
